@@ -5,7 +5,7 @@ import numpy as np
 from sfsimodels.models.abstract_models import PhysicalObject
 from sfsimodels.models import SeismicHazard, Foundation, Soil
 from sfsimodels.models.material import Concrete
-from sfsimodels.exceptions import ModelError
+from sfsimodels.exceptions import ModelError, deprecation
 from sfsimodels import functions as sf
 
 
@@ -31,7 +31,7 @@ class Building(PhysicalObject):
         self._n_storeys = n_storeys
         if not hasattr(self, "inputs"):
             self.inputs = []
-        self.inputs += [
+        self._extra_class_variables = [
                 "id",
                 "name",
                 "type",
@@ -40,6 +40,7 @@ class Building(PhysicalObject):
                 'interstorey_heights',
                 'storey_masses'
             ]
+        self.inputs += self._extra_class_variables
         self.all_parameters = self.inputs + [
             "n_storeys"
         ]
@@ -179,17 +180,58 @@ class Section(PhysicalObject):  # not used?
         self._width = value
 
 
+class Element(PhysicalObject):
+    section_lengths = None
+
+    def __init__(self):
+        self.sections = [Section()]
+
+    @property
+    def s(self):
+        return self.sections
+
+    def set_section_prop(self, prop, prop_value, sections=None):
+        if sections is None:
+            sections = range(len(self.sections))
+        for sect_i in sections:
+            setattr(self.sections[sect_i], prop, prop_value)
+
+    def add_inputs_to_section(self, props, sections=None):
+        if sections is None:
+            sections = range(len(self.sections))
+        for sect_i in sections:
+            self.sections[sect_i].inputs += props
+
+    def split_into_multiple(self, lengths):
+        section = self.sections[0]
+        self.section_lengths = []
+        self.sections = []
+        for length in lengths:
+            self.section_lengths.append(length)
+            self.sections.append(section.deepcopy())
+
+    def get_section_prop(self, prop, section_i=0):
+        return getattr(self.sections[section_i], prop)
+
+    def to_dict(self, extra=()):
+        output = []
+        for i in range(len(self.sections)):
+            output.append(self.sections[i].to_dict(extra=extra))
+        return []
+
+
 class Frame(object):
     _bay_lengths = None
 
     def __init__(self, n_storeys, n_bays):
         if not hasattr(self, "inputs"):
             self.inputs = []
-        self.inputs += [
+        self._extra_class_variables = [
             "beams",
             "columns",
             "bay_lengths"
         ]
+        self.inputs += self._extra_class_variables
         self._n_storeys = n_storeys
         self._n_bays = n_bays
         self._allocate_beams_and_columns()
@@ -198,7 +240,7 @@ class Frame(object):
         outputs = OrderedDict()
         skip_list = []
         # skip_list = ["beams", "columns"]  # TODO: uncomment this
-        skip_list = ["beams"]
+        # skip_list = ["beams"]
         full_inputs = self.inputs + list(extra)
         for item in full_inputs:
             if item not in skip_list:
@@ -235,8 +277,8 @@ class Frame(object):
         return ["frame"]
 
     def _allocate_beams_and_columns(self):
-        self._beams = [[Section() for i in range(self.n_bays)] for ss in range(self.n_storeys)]
-        self._columns = [[Section() for i in range(self.n_cols)] for ss in range(self.n_storeys)]
+        self._beams = [[Element() for i in range(self.n_bays)] for ss in range(self.n_storeys)]
+        self._columns = [[Element() for i in range(self.n_cols)] for ss in range(self.n_storeys)]
 
     @property
     def beams(self):
@@ -284,7 +326,7 @@ class Frame(object):
             raise ModelError("beam depths does not match number of bays (%i)." % self.n_bays)
         for ss in range(self.n_storeys):
             for i in range(self.n_bays):
-                setattr(self._beams[ss][i], prop, values[0][i])
+                self._beams[ss][i].set_section_prop(prop, values[0][i])
 
     def set_column_prop(self, prop, values, repeat="up"):
         """
@@ -304,7 +346,7 @@ class Frame(object):
             raise ModelError("column props does not match n_cols (%i)." % self.n_cols)
         for ss in range(self.n_storeys):
             for i in range(self.n_cols):
-                setattr(self._columns[ss][i], prop, values[0][i])
+                self._columns[ss][i].set_section_prop(prop, values[0][i])
 
     def beams_at_storey(self, storey):
         return self._beams[storey - 1]
@@ -315,7 +357,7 @@ class Frame(object):
         for ss in range(self.n_storeys):
             beam_depths.append([])
             for i in range(self.n_bays):
-                beam_depths[ss].append(self.beams[ss][i].depth)
+                beam_depths[ss].append(self.beams[ss][i].get_section_prop("depth"))
         return np.array(beam_depths)
 
     @property
@@ -324,7 +366,7 @@ class Frame(object):
         for ss in range(self.n_storeys):
             beam_widths.append([])
             for i in range(self.n_bays):
-                beam_widths[ss].append(self.beams[ss][i].width)
+                beam_widths[ss].append(self.beams[ss][i].get_section_prop("width"))
         return np.array(beam_widths)
 
     @property
@@ -333,7 +375,7 @@ class Frame(object):
         for ss in range(self.n_storeys):
             column_depths.append([])
             for i in range(self.n_cols):
-                column_depths[ss].append(self.columns[ss][i].width)
+                column_depths[ss].append(self.columns[ss][i].get_section_prop("depth"))
         return np.array(column_depths)
 
     @property
@@ -342,7 +384,7 @@ class Frame(object):
         for ss in range(self.n_storeys):
             column_widths.append([])
             for i in range(self.n_cols):
-                column_widths[ss].append(self.columns[ss][i].width)
+                column_widths[ss].append(self.columns[ss][i].get_section_prop("width"))
         return np.array(column_widths)
 
     @property
@@ -356,24 +398,24 @@ class Frame(object):
         self._bay_lengths = np.array(bay_lengths)
 
 
-class FrameBuilding(Frame, Building):
+class BuildingFrame(Frame, Building):
     _n_seismic_frames = None
     _n_gravity_frames = None
-    type = "frame_building"
-    _extra_class_inputs = ["n_seismic_frames",
-                           "n_gravity_frames"]
+    type = "building_frame"
 
     def __init__(self, n_storeys, n_bays):
         Frame.__init__(self, n_storeys, n_bays)
         Building.__init__(self, n_storeys)
-        # super(FrameBuilding, self).__init__(n_storeys, n_bays)  # run parent class initialiser function
+        # super(BuildingFrame, self).__init__(n_storeys, n_bays)  # run parent class initialiser function
+        self._extra_class_inputs = ["n_seismic_frames",
+                           "n_gravity_frames"]
         self.inputs = self.inputs + self._extra_class_inputs
         # Frame.__init__(self, n_storeys, n_bays)
         # Building.__init__(self, n_storeys, n_bays)
 
     @property
     def ancestor_types(self):
-        return super(FrameBuilding, self).ancestor_types + ["frame_building"]
+        return super(BuildingFrame, self).ancestor_types + ["building_frame"]
 
     @property
     def n_seismic_frames(self):
@@ -392,9 +434,15 @@ class FrameBuilding(Frame, Building):
         self._n_gravity_frames = value
 
 
-class FrameBuilding2D(Frame, Building):
+class FrameBuilding(BuildingFrame):
+    def __init__(self, n_storeys, n_bays):
+        deprecation("FrameBuilding is deprecated, use BuildingFrame.")
+        super(FrameBuilding, self).__init__(n_storeys, n_bays)
+
+
+class BuildingFrame2D(Frame, Building):
     _extra_class_inputs = []
-    type = "frame_building_2D"
+    type = "building_frame2D"
 
     def __init__(self, n_storeys, n_bays):
         Frame.__init__(self, n_storeys, n_bays)
@@ -404,7 +452,7 @@ class FrameBuilding2D(Frame, Building):
 
     @property
     def ancestor_types(self):
-        return ["physical_object", "frame", "building"] + ["frame_building_2D"]  # TODO: improve this logic
+        return ["physical_object", "frame", "building"] + ["building_frame2D"]  # TODO: improve this logic
 
     # def to_dict(self, extra=()):
     #     outputs = OrderedDict()
@@ -434,15 +482,17 @@ class FrameBuilding2D(Frame, Building):
             for cc in range(self.n_cols):
                 # build a hash string of the section inputs to check uniqueness
                 parts = []
-                for item in self.columns[ss][cc].inputs:
-                    parts.append(item)
-                    parts.append(str(getattr(self.columns[ss][cc], item)))
-                p_str = "-".join(parts)
-                if p_str not in column_sections:
-                    column_sections[p_str] = self.columns[ss][cc].to_dict(extra)
-                    column_section_count += 1
-                    column_sections[p_str]["id"] = column_section_count
-                column_section_ids[ss].append(column_section_count)
+                column_section_ids[ss].append([])
+                for sect_i in range(len(self.columns[ss][cc].sections)):
+                    for item in self.columns[ss][cc].sections[sect_i].inputs:
+                        parts.append(item)
+                        parts.append(str(self.columns[ss][cc].get_section_prop(item, section_i=sect_i)))
+                    p_str = "-".join(parts)
+                    if p_str not in column_sections:
+                        column_sections[p_str] = self.columns[ss][cc].sections[sect_i].to_dict(extra)
+                        column_section_count += 1
+                        column_sections[p_str]["id"] = column_section_count
+                    column_section_ids[ss][cc].append(column_section_count)
 
         beam_sections = OrderedDict()
         beam_section_ids = []
@@ -452,15 +502,17 @@ class FrameBuilding2D(Frame, Building):
             for bb in range(self.n_bays):
                 # build a hash string of the section inputs to check uniqueness
                 parts = []
-                for item in self.beams[ss][bb].inputs:
-                    parts.append(item)
-                    parts.append(str(getattr(self.beams[ss][bb], item)))
-                p_str = "-".join(parts)
-                if p_str not in beam_sections:
-                    beam_sections[p_str] = self.beams[ss][bb].to_dict(extra)
-                    beam_section_count += 1
-                    beam_sections[p_str]["id"] = beam_section_count
-                beam_section_ids[ss].append(beam_section_count)
+                beam_section_ids[ss].append([])
+                for sect_i in range(len(self.beams[ss][bb].sections)):
+                    for item in self.beams[ss][bb].sections[sect_i].inputs:
+                        parts.append(item)
+                        parts.append(str(self.beams[ss][bb].get_section_prop(item, section_i=sect_i)))
+                    p_str = "-".join(parts)
+                    if p_str not in beam_sections:
+                        beam_sections[p_str] = self.beams[ss][bb].sections[sect_i].to_dict(extra)
+                        beam_section_count += 1
+                        beam_sections[p_str]["id"] = beam_section_count
+                    beam_section_ids[ss][bb].append(beam_section_count)
 
         outputs["column_section_ids"] = column_section_ids
         outputs["beam_section_ids"] = beam_section_ids
@@ -473,51 +525,62 @@ class FrameBuilding2D(Frame, Building):
         return outputs
 
 
-class WallBuilding(Building):
+class FrameBuilding2D(BuildingFrame2D):
+    def __init__(self, n_storeys, n_bays):
+        deprecation("FrameBuilding2D class is deprecated, use BuildingFrame2D.")
+        super(FrameBuilding2D, self).__init__(n_storeys, n_bays)
+
+
+class BuildingWall(Building):  # new name
     n_walls = 1
     wall_depth = 0.0  # m
     wall_width = 0.0  # m
-    type = "wall_building"
-    _extra_class_inputs = [
-        "n_walls",
-        "wall_depth",
-        "wall_width"
-    ]
+    type = "building_wall"
 
     def __init__(self, n_storeys):
-        super(WallBuilding, self).__init__(n_storeys)  # run parent class initialiser function
-        self.inputs = self.inputs + self._extra_class_inputs
+        super(BuildingWall, self).__init__(n_storeys)  # run parent class initialiser function
+        self._extra_class_inputs = [
+            "n_walls",
+            "wall_depth",
+            "wall_width"
+        ]
+        self.inputs += self._extra_class_inputs
 
     @property
     def ancestor_types(self):
-        return super(WallBuilding, self).ancestor_types + ["wall_building"]
+        return super(BuildingWall, self).ancestor_types + ["building_wall"]
 
 
-class Structure(PhysicalObject):
+class WallBuilding(BuildingWall):
+    def __init__(self, n_storeys):
+        deprecation("WallBuilding class is deprecated, use BuildingWall.")
+        super(WallBuilding, self).__init__(n_storeys)
+
+
+class BuildingSDOF(PhysicalObject):
     """
     An object to describe structures.
     """
     _id = None
     name = None
     base_type = "building"
-    type = "structure"
+    type = "sdof"
     _h_eff = None
     _mass_eff = None
     _t_fixed = None
     _mass_ratio = None
 
-    inputs = [
-        "id",
-        "name",
-        "base_type",
-        "type",
-        "h_eff",
-        "mass_eff",
-        "t_fixed",
-        "mass_ratio"
-    ]
-
     def __init__(self, g=9.8):
+        self.inputs = [
+            "id",
+            "name",
+            "base_type",
+            "type",
+            "h_eff",
+            "mass_eff",
+            "t_fixed",
+            "mass_ratio"
+        ]
         self._g = g
 
     def to_dict(self):
@@ -534,7 +597,7 @@ class Structure(PhysicalObject):
 
     @property
     def ancestor_types(self):
-        return super(Structure, self).ancestor_types + ["structure"]
+        return super(BuildingSDOF, self).ancestor_types + ["sdof"]
 
     @property
     def id(self):
@@ -593,8 +656,14 @@ class Structure(PhysicalObject):
         return self.mass_eff / self.mass_ratio * self._g
 
 
+class Structure(BuildingSDOF):
+    def __init__(self, g=9.8):
+        deprecation("Structure class is deprecated, use BuildingSDOF.")
+        super(Structure, self).__init__(g=g)
+
+
 class SoilStructureSystem(PhysicalObject):
-    bd = Structure()
+    bd = BuildingSDOF()
     fd = Foundation()
     sp = Soil()
     hz = SeismicHazard()
@@ -604,6 +673,6 @@ class SoilStructureSystem(PhysicalObject):
 
 
 if __name__ == '__main__':
-    print(FrameBuilding2D.__mro__)
-    a = FrameBuilding2D(1, 2)
+    print(BuildingFrame2D.__mro__)
+    a = BuildingFrame2D(1, 2)
     print(a.n_bays)
