@@ -43,7 +43,7 @@ class Soil(PhysicalObject):
     def __init__(self, pw=9800):
         self._pw = pw  # specific weight of water
         self.stack = []
-        self.inputs = [  # TODO: should run init method of parent then take inputs properly
+        self._extra_class_inputs = [
             "id",
             "name",
             "base_type",
@@ -66,6 +66,9 @@ class Soil(PhysicalObject):
             "plasticity_index",
             "permeability"
         ]
+        if not hasattr(self, "inputs"):
+            self.inputs = []
+        self.inputs += self._extra_class_inputs
 
     def to_dict(self):
         """
@@ -191,6 +194,11 @@ class Soil(PhysicalObject):
         """The specific gravity of the soil"""
         return self._specific_gravity
 
+    @property
+    def pw(self):
+        """Specific weight of water"""
+        return self._pw
+
     @property
     def saturation(self):
         """The current saturation of the soil"""
@@ -684,44 +692,40 @@ class Soil(PhysicalObject):
             return None
 
 
-class CriticalSoil(Soil):
+class SoilCritical(Soil):
     # critical state parameters
     e_cr0 = 0.0
     p_cr0 = 0.0
     lamb_crl = 0.0
-    type = "critical_soil"
-    _extra_class_inputs = [
-        "e_cr0",
-        "p_cr0",
-        "lamb_crl"
-        ]
+    type = "soil_critical"
 
-    def __init__(self):
-        super(CriticalSoil, self).__init__()  # run parent class initialiser function
+    def __init__(self, pw=9800):
+        super(SoilCritical, self).__init__(pw=pw)  # run parent class initialiser function
+        self._extra_class_inputs = ["e_cr0", "p_cr0", "lamb_crl"]
         self.inputs = self.inputs + self._extra_class_inputs
 
     @property
     def ancestor_types(self):
-        return super(CriticalSoil, self).ancestor_types + [self.type]
+        return super(SoilCritical, self).ancestor_types + [self.type]
 
     def e_critical(self, p):
         p = float(p)
         return self.e_cr0 - self.lamb_crl * np.log(p / self.p_cr0)
 
 
-class StressDependentSoil(Soil):
+class SoilStressDependent(Soil):
     _g0_mod = None
-    _p_atm = 10100.0  # kPa
-    type = "stress_dependent_soil"
-    _extra_class_inputs = ["g0_mod", "p_atm"]
+    _p_atm = 101000.0  # Pa
+    type = "soil_stress_dependent"
 
-    def __init__(self):
-        super(StressDependentSoil, self).__init__()
+    def __init__(self, pw=9800):
+        super(SoilStressDependent, self).__init__(pw=pw)
+        self._extra_class_inputs = ["g0_mod", "p_atm"]
         self.inputs = self.inputs + self._extra_class_inputs
 
     @property
     def ancestor_types(self):
-        return super(StressDependentSoil, self).ancestor_types + [self.type]
+        return super(SoilStressDependent, self).ancestor_types + [self.type]
 
     @property
     def g0_mod(self):
@@ -741,8 +745,12 @@ class StressDependentSoil(Soil):
         value = clean_float(value)
         self._p_atm = value
 
-    def g_mod_at_eff_stress(self, sigma_v_eff):
-        return self.g0_mod * self.p_atm * (sigma_v_eff / self.p_atm) ** 0.5
+    def g_mod_at_v_eff_stress(self, sigma_v_eff):
+        k0 = 1 - np.sin(self.phi_r)
+        return self.g0_mod * self.p_atm * (sigma_v_eff * (1 + 2 * k0) / 3 / self.p_atm) ** 0.5
+
+    def g_mod_at_m_eff_stress(self, sigma_m_eff):
+        return self.g0_mod * self.p_atm * (sigma_m_eff / self.p_atm) ** 0.5
 
 
 class SoilLayer(Soil):  # not used
@@ -869,8 +877,19 @@ class SoilProfile(PhysicalObject):
     def layers(self):
         return self._layers
 
-    def remove_layer(self, depth):
-        del self._layers[depth]
+    def remove_layer_at_depth(self, depth):
+        try:
+            del self._layers[depth]
+        except KeyError:
+            raise KeyError("Depth: {0} not found in {1}".format(depth, list(self.layers.keys())))
+
+    def remove_layer(self, layer_int):
+        key = list(self._layers.keys())[layer_int - 1]
+        del self._layers[key]
+
+    def replace_layer(self, layer_int, soil):
+        key = list(self._layers.keys())[layer_int - 1]
+        self._layers[key] = soil
 
     def layer(self, index):
         index = int(index)
